@@ -1,7 +1,7 @@
 ##    _____  _____
 ##   |  __ \|  __ \    AUTHOR: Pedro Rivero
 ##   | |__) | |__) |   ---------------------------------
-##   |  ___/|  _  /    DATE: February 23, 2021
+##   |  ___/|  _  /    DATE: May 19, 2021
 ##   | |    | | \ \    ---------------------------------
 ##   |_|    |_|  \_\   https://github.com/pedrorrivero
 ##
@@ -21,15 +21,17 @@
 ## limitations under the License.
 
 import struct
-from typing import Any, Callable, Final, List, Optional, Tuple, Union
+from typing import Any, Callable, Final, List, Optional, Tuple
 
-from numpy import float64, uint32, uint64
 from qiskit import BasicAer, QuantumCircuit, execute
 from qiskit.providers import Backend, Job, Provider
 from qiskit.providers.ibmq import IBMQError, least_busy
 from qiskit.providers.models import BackendConfiguration
 from qiskit.result import Counts, Result
-from randomgen import UserBitGenerator
+
+from .caches import BasicCache as BitCache
+from .errors import raise_future_warning
+from .quantum_bit_generator import QuantumBitGenerator
 
 ###############################################################################
 ## CUSTOM TYPES
@@ -37,138 +39,10 @@ from randomgen import UserBitGenerator
 BackendFilter = Callable[[Backend], bool]
 
 
-###############################################################################
-## BIT CACHE
-###############################################################################
-class BitCache:
-    """
-    BitCache first in, first out (FIFO) data structure.
-    """
-
-    def __init__(self) -> None:
-        self._cache: str = ""
-        self.size: int = 0
-
-    ########################## STATIC/CLASS METHODS ##########################
-    @staticmethod
-    def isbitstring(bitstring: str) -> bool:
-        """
-        Returns `True` if the input str is a bitstring, `False` otherwise.
-
-        PARAMETERS
-        ----------
-        bitstring: str
-            The string to check.
-
-        RETURNS
-        -------
-        out: bool
-            `True` if input str is bitstring, `False` otherwise.
-
-        RAISES
-        ------
-        TypeError
-            If input bitstring is not str.
-        """
-        if not isinstance(bitstring, str):
-            raise TypeError(f"Invalid bitstring type '{type(bitstring)}'")
-        b = {"0", "1"}
-        s = set(bitstring)
-        return s.issubset(b)
-
-    ############################# PUBLIC METHODS #############################
-    def dump(self) -> str:
-        """
-        Outputs all the contents in the cache without erasing.
-
-        RETURNS
-        -------
-        out: str
-            Cache contents.
-        """
-        return self._cache
-
-    def flush(self) -> bool:
-        """
-        Erases the cache.
-
-        RETURNS
-        -------
-        out: bool
-            `True` if succeeds, `False` otherwise.
-        """
-        self._cache = ""
-        self.size = 0
-        return True
-
-    def pop(self, n: int) -> str:
-        """
-        Returns a size `n` bitstring removing it from the top of the cache.
-
-        PARAMETERS
-        ----------
-        n: int
-            Number of bits to retrieve
-
-        RETURNS
-        -------
-        out: str
-            Size `n` bitstring.
-
-        RAISES
-        ------
-        ValueError
-            If input is greater than cache size, or less than one.
-        """
-        if n < 1:
-            raise ValueError("Input number of bits must be greater than zero")
-        elif n > self.size:
-            raise ValueError("Insufficient cache size")
-        bitstring: str = self._cache[:n]
-        self._cache = self._cache[n:]
-        self.size -= n if n < self.size else self.size
-        return bitstring
-
-    def push(self, bitstring: str) -> bool:
-        """
-        Inserts bitstring at the end of the cache.
-
-        PARAMETERS
-        ----------
-        bitstring: str
-            The bitstring to insert.
-
-        RETURNS
-        -------
-        out: bool
-            `True` if succeeds, `False` otherwise.
-
-        RAISES
-        ------
-        TypeError (isbitstring)
-            If input bitstring is not str
-        ValueError
-            If input bitstring is not a valid bitstring
-        """
-        if not BitCache.isbitstring(bitstring):
-            raise ValueError(f"Invalid bitstring value '{bitstring}'")
-        self._cache += bitstring
-        self.size += len(bitstring)
-        return True
-
-    ############################ PUBLIC PROPERTIES ############################
-    @property
-    def state(self) -> dict:
-        """
-        The state of the BitCache object.
-        """
-        return {"size": self.size}
-
-
 ###############################################################################
 ## QISKIT BIT GENERATOR
 ###############################################################################
-class QiskitBitGenerator(UserBitGenerator):
+class QiskitBitGenerator(QuantumBitGenerator):
     """
     A quantum random bit-generator based on Qiskit, which can interface with
     NumPy's random module (e.g. to instantiate Generator objects). It
@@ -235,6 +109,11 @@ class QiskitBitGenerator(UserBitGenerator):
         max_bits_per_request: int = 0,
         ISRAW32: bool = False,
     ) -> None:
+        raise_future_warning(
+            "QiskitBitGenerator",
+            "1.0.0",
+            "QuantumBitGenerator and QiskitPlatform",
+        )
         if backend:
             provider = None
         elif provider:
@@ -248,9 +127,9 @@ class QiskitBitGenerator(UserBitGenerator):
         self._backend: Backend = backend
         self._backend_filter: Optional[BackendFilter] = backend_filter
         self._set_mbpr(max_bits_per_request)
-        self._ISRAW32: Final[bool] = ISRAW32
+        self._ISRAW32: Final[bool] = ISRAW32  # type: ignore
         self._bitcache: BitCache = BitCache()
-        super().__init__(
+        super(QuantumBitGenerator, self).__init__(
             bits=self.BITS,
             next_raw=self._next_raw,
             next_32=self._next_32,
@@ -313,26 +192,7 @@ class QiskitBitGenerator(UserBitGenerator):
         return least_busy(backends)
 
     ############################# PUBLIC METHODS #############################
-    def dump_cache(self, flush: bool = False) -> str:
-        """
-        Returns all the contents stored in the cache.
-
-        PARAMETERS
-        ----------
-        flush: bool
-            If `True` erase the cache after dumping.
-
-        RETURNS
-        -------
-        out: str
-            The bitstring stored in cache.
-        """
-        bitstring: str = self._bitcache.dump()
-        if flush:
-            self._bitcache.flush()
-        return bitstring
-
-    def flush_cache(self) -> bool:
+    def flush_cache(self) -> None:
         """
         Erase the cache.
 
@@ -341,9 +201,9 @@ class QiskitBitGenerator(UserBitGenerator):
         out: bool
             `True` if succeeds, `False` otherwise.
         """
-        return self._bitcache.flush()
+        self._bitcache.flush()
 
-    def load_cache(self, bitstring: str, flush: bool = False) -> bool:
+    def load_cache(self, bitstring: str, flush: bool = False) -> None:
         """
         Load cache contents from bitstring.
 
@@ -367,10 +227,10 @@ class QiskitBitGenerator(UserBitGenerator):
             If input bitstring is not a valid bitstring
         """
         if flush:
-            return self._bitcache.flush() and self._bitcache.push(bitstring)
-        return self._bitcache.push(bitstring)
+            self._bitcache.flush()
+        self._bitcache.push(bitstring)
 
-    def random_bitstring(self, n_bits: int = 0) -> str:
+    def random_bitstring(self, n_bits: int = 0) -> str:  # type: ignore
         """
         Returns a random bitstring of a given lenght. If less than one it
         defaults to the raw number of bits for the instance QiskitBitGenerator
@@ -392,36 +252,7 @@ class QiskitBitGenerator(UserBitGenerator):
             self._fetch_random_bits()
         return self._bitcache.pop(n_bits)
 
-    def random_double(self, n: float = 1) -> float:
-        """
-        Returns a random double from a uniform distribution in the range
-        [0,n). Defaults to [0,1).
-
-        PARAMETERS
-        ----------
-        n: float
-            Size of the range [0,n) from which to draw the random number.
-
-        RETURNS
-        -------
-        out: float
-            Random float in the range [0,n).
-
-        COPYRIGHT NOTICE
-        ----------------
-        Source: https://github.com/ozanerhansha/qRNG
-        License: GNU GENERAL PUBLIC LICENSE VERSION 3
-        Changes:
-            - Add static type hints
-            - Limit range to [0,n) instead of [min,max) and add default
-            - Replace call to original get_random_int64
-        """
-        unpacked = 0x3FF0000000000000 | self.random_uint(64) >> 12
-        packed = struct.pack("Q", unpacked)
-        value: float = struct.unpack("d", packed)[0] - 1.0
-        return value * n
-
-    def random_uint(self, n_bits: int = 0) -> int:
+    def random_uint(self, n_bits: int = 0) -> int:  # type: ignore
         """
         Returns a random unsigned int of a given size in bits.
 
@@ -535,15 +366,6 @@ class QiskitBitGenerator(UserBitGenerator):
         return True
 
     ############################ PUBLIC PROPERTIES ############################
-    @property
-    def BITS(self) -> int:
-        """
-        Either 32 or 64. The number of bits output by NumPy's `random_raw()`
-        method. Final, it cannot be modified after instantiation through the
-        ISRAW32 parameter.
-        """
-        return 32 if self._ISRAW32 else 64
-
     @property
     def state(self) -> dict:
         """
@@ -556,7 +378,7 @@ class QiskitBitGenerator(UserBitGenerator):
             "dynamic_backend": {
                 "filter": "Custom" if self._backend_filter else "Default",
             },
-            "bitcache": self._bitcache.state,
+            "bitcache": {"size": self._bitcache.size},
         }
         if not self._provider:
             s.pop("dynamic_backend")
@@ -622,7 +444,7 @@ class QiskitBitGenerator(UserBitGenerator):
             and backend_config["n_qubits"]
             else 1
         )
-        max_bits_per_request: int = self._max_bits_per_request
+        max_bits_per_request: int = self._max_bits_per_request or 0
         if max_bits_per_request > n_qubits:
             experiments = min(
                 experiments,
@@ -651,48 +473,3 @@ class QiskitBitGenerator(UserBitGenerator):
     def _shots(self) -> int:
         n_qubits, shots, experiments = self._job_partition
         return shots
-
-    ############################# NUMPY INTERFACE #############################
-    @property
-    def _next_raw(self) -> Callable[[Any], Union[uint32, uint64]]:
-        """
-        A callable that returns either 64 or 32 random bits. It must accept
-        a single input which is a void pointer to a memory address.
-        """
-        return self._next_32 if self._ISRAW32 else self._next_64
-
-    @property
-    def _next_32(self) -> Callable[[Any], uint32]:
-        """
-        A callable with the same signature as as next_raw that always returns
-        a random numpy 32-bit unsigned int.
-        """
-
-        def next_32(void_p: Any) -> uint32:
-            return uint32(self.random_uint(32))
-
-        return next_32
-
-    @property
-    def _next_64(self) -> Callable[[Any], uint64]:
-        """
-        A callable with the same signature as as next_raw that always returns
-        a random numpy 64-bit unsigned int.
-        """
-
-        def next_64(void_p: Any) -> uint64:
-            return uint64(self.random_uint(64))
-
-        return next_64
-
-    @property
-    def _next_double(self) -> Callable[[Any], float64]:
-        """
-        A callable with the same signature as as next_raw that always return
-        a random double in [0,1).
-        """
-
-        def next_double(void_p: Any) -> float64:
-            return float64(self.random_double(1))
-
-        return next_double
